@@ -1,31 +1,23 @@
 /*
- * "$Id: runloop.c 6456 2007-04-16 18:33:38Z mike $"
+ * "$Id: runloop.c 8132 2008-11-14 21:02:37Z mike $"
  *
- *   Common run loop API for the Common UNIX Printing System (CUPS).
+ *   Common run loop APIs for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 2006 by Easy Software Products, all rights reserved.
+ *   Copyright 2007-2008 by Apple Inc.
+ *   Copyright 2006-2007 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
  *   "LICENSE" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  *   This file is subject to the Apple OS-Developed Software exception.
  *
  * Contents:
  *
- *   backendRunLoop() - Read and write print and back-channel data.
+ *   backendDrainOutput() - Drain pending print data to the device.
+ *   backendRunLoop()     - Read and write print and back-channel data.
  */
 
 /*
@@ -40,14 +32,119 @@
 #endif /* __hpux */
 
 
+/*
+ * 'backendDrainOutput()' - Drain pending print data to the device.
+ */
+
+int					/* O - 0 on success, -1 on error */
+backendDrainOutput(int print_fd,	/* I - Print file descriptor */
+                   int device_fd)	/* I - Device file descriptor */
+{
+  int		nfds;			/* Maximum file descriptor value + 1 */
+  fd_set	input;			/* Input set for reading */
+  ssize_t	print_bytes,		/* Print bytes read */
+		bytes;			/* Bytes written */
+  char		print_buffer[8192],	/* Print data buffer */
+		*print_ptr;		/* Pointer into print data buffer */
+  struct timeval timeout;		/* Timeout for read... */
+
+
+  fprintf(stderr, "DEBUG: backendDrainOutput(print_fd=%d, device_fd=%d)\n",
+          print_fd, device_fd);
+
+ /*
+  * Figure out the maximum file descriptor value to use with select()...
+  */
+
+  nfds = (print_fd > device_fd ? print_fd : device_fd) + 1;
+
+ /*
+  * Now loop until we are out of data from print_fd...
+  */
+
+  for (;;)
+  {
+   /*
+    * Use select() to determine whether we have data to copy around...
+    */
+
+    FD_ZERO(&input);
+    FD_SET(print_fd, &input);
+
+    timeout.tv_sec  = 0;
+    timeout.tv_usec = 0;
+
+    if (select(nfds, &input, NULL, NULL, &timeout) < 0)
+      return (-1);
+
+    if (!FD_ISSET(print_fd, &input))
+      return (0);
+
+    if ((print_bytes = read(print_fd, print_buffer,
+			    sizeof(print_buffer))) < 0)
+    {
+     /*
+      * Read error - bail if we don't see EAGAIN or EINTR...
+      */
+
+      if (errno != EAGAIN || errno != EINTR)
+      {
+	perror("ERROR: Unable to read print data");
+	return (-1);
+      }
+
+      print_bytes = 0;
+    }
+    else if (print_bytes == 0)
+    {
+     /*
+      * End of file, return...
+      */
+
+      return (0);
+    }
+
+    fprintf(stderr, "DEBUG: Read %d bytes of print data...\n",
+	    (int)print_bytes);
+
+    for (print_ptr = print_buffer; print_bytes > 0;)
+    {
+      if ((bytes = write(device_fd, print_ptr, print_bytes)) < 0)
+      {
+       /*
+        * Write error - bail if we don't see an error we can retry...
+	*/
+
+        if (errno != ENOSPC && errno != ENXIO && errno != EAGAIN &&
+	    errno != EINTR && errno != ENOTTY)
+	{
+	  _cupsLangPrintf(stderr, _("ERROR: Unable to write print data: %s\n"),
+			  strerror(errno));
+	  return (-1);
+	}
+      }
+      else
+      {
+        fprintf(stderr, "DEBUG: Wrote %d bytes of print data...\n", (int)bytes);
+
+        print_bytes -= bytes;
+	print_ptr   += bytes;
+      }
+    }
+  }
+}
+
+
 /*
  * 'backendRunLoop()' - Read and write print and back-channel data.
  */
 
 ssize_t					/* O - Total bytes on success, -1 on error */
-backendRunLoop(int print_fd,		/* I - Print file descriptor */
-               int device_fd,		/* I - Device file descriptor */
-	       int use_bc)		/* I - Use back-channel? */
+backendRunLoop(
+    int  print_fd,			/* I - Print file descriptor */
+    int  device_fd,			/* I - Device file descriptor */
+    int  use_bc,			/* I - Use back-channel? */
+    void (*side_cb)(int, int, int))	/* I - Side-channel callback */
 {
   int		nfds;			/* Maximum file descriptor value + 1 */
   fd_set	input,			/* Input set for reading */
@@ -66,8 +163,10 @@ backendRunLoop(int print_fd,		/* I - Print file descriptor */
 #endif /* HAVE_SIGACTION && !HAVE_SIGSET */
 
 
-  fprintf(stderr, "DEBUG: backendRunLoop(print_fd=%d, device_fd=%d, use_bc=%d)\n",
-          print_fd, device_fd, use_bc);
+  fprintf(stderr,
+          "DEBUG: backendRunLoop(print_fd=%d, device_fd=%d, use_bc=%d, "
+	  "side_cb=%p)\n",
+          print_fd, device_fd, use_bc, side_cb);
 
  /*
   * If we are printing data from a print driver on stdin, ignore SIGTERM
@@ -113,12 +212,14 @@ backendRunLoop(int print_fd,		/* I - Print file descriptor */
       FD_SET(print_fd, &input);
     if (use_bc)
       FD_SET(device_fd, &input);
+    if (!print_bytes && side_cb)
+      FD_SET(CUPS_SC_FD, &input);
 
     FD_ZERO(&output);
-    if (print_bytes || !use_bc)
+    if (print_bytes || (!use_bc && !side_cb))
       FD_SET(device_fd, &output);
 
-    if (use_bc)
+    if (use_bc || side_cb)
     {
       if (select(nfds, &input, &output, NULL, NULL) < 0)
       {
@@ -129,15 +230,36 @@ backendRunLoop(int print_fd,		/* I - Print file descriptor */
 	if (errno == ENXIO && offline != 1)
 	{
 	  fputs("STATE: +offline-error\n", stderr);
-	  fputs("INFO: Printer is currently off-line.\n", stderr);
+	  _cupsLangPuts(stderr, _("INFO: Printer is currently off-line.\n"));
 	  offline = 1;
 	}
+	else if (errno == EINTR && total_bytes == 0)
+	{
+	  fputs("DEBUG: Received an interrupt before any bytes were "
+	        "written, aborting!\n", stderr);
+          return (0);
+	}
 
 	sleep(1);
 	continue;
       }
     }
 
+   /*
+    * Check if we have a side-channel request ready...
+    */
+
+    if (side_cb && FD_ISSET(CUPS_SC_FD, &input))
+    {
+     /*
+      * Do the side-channel request, then start back over in the select
+      * loop since it may have read from print_fd...
+      */
+
+      (*side_cb)(print_fd, device_fd, use_bc);
+      continue;
+    }
+
    /*
     * Check if we have back-channel data ready...
     */
@@ -151,6 +273,12 @@ backendRunLoop(int print_fd,		/* I - Print file descriptor */
 	        CUPS_LLCAST bc_bytes);
         cupsBackChannelWrite(bc_buffer, bc_bytes, 1.0);
       }
+      else if (bc_bytes < 0 && errno != EAGAIN && errno != EINTR)
+      {
+        fprintf(stderr, "DEBUG: Error reading back-channel data: %s\n",
+	        strerror(errno));
+	use_bc = 0;
+      }
     }
 
    /*
@@ -206,8 +334,8 @@ backendRunLoop(int print_fd,		/* I - Print file descriptor */
 	{
 	  if (paperout != 1)
 	  {
-	    fputs("ERROR: Out of paper!\n", stderr);
 	    fputs("STATE: +media-empty-error\n", stderr);
+	    _cupsLangPuts(stderr, _("ERROR: Out of paper!\n"));
 	    paperout = 1;
 	  }
         }
@@ -216,13 +344,14 @@ backendRunLoop(int print_fd,		/* I - Print file descriptor */
 	  if (offline != 1)
 	  {
 	    fputs("STATE: +offline-error\n", stderr);
-	    fputs("INFO: Printer is currently off-line.\n", stderr);
+	    _cupsLangPuts(stderr, _("INFO: Printer is currently off-line.\n"));
 	    offline = 1;
 	  }
 	}
 	else if (errno != EAGAIN && errno != EINTR && errno != ENOTTY)
 	{
-	  perror("ERROR: Unable to write print data");
+	  fprintf(stderr, _("ERROR: Unable to write print data: %s\n"),
+	          strerror(errno));
 	  return (-1);
 	}
       }
@@ -237,7 +366,7 @@ backendRunLoop(int print_fd,		/* I - Print file descriptor */
 	if (offline)
 	{
 	  fputs("STATE: -offline-error\n", stderr);
-	  fputs("INFO: Printer is now on-line.\n", stderr);
+	  _cupsLangPuts(stderr, _("INFO: Printer is now on-line.\n"));
 	  offline = 0;
 	}
 
@@ -259,5 +388,5 @@ backendRunLoop(int print_fd,		/* I - Print file descriptor */
 
 
 /*
- * End of "$Id: runloop.c 6456 2007-04-16 18:33:38Z mike $".
+ * End of "$Id: runloop.c 8132 2008-11-14 21:02:37Z mike $".
  */
