@@ -1,8 +1,9 @@
 /*
- * "$Id: scsi-linux.c,v 1.5 2005/01/03 19:29:44 mike Exp $"
+ * "$Id: scsi-linux.c 7721 2008-07-11 22:48:49Z mike $"
  *
  *   Linux SCSI printer support for the Common UNIX Printing System (CUPS).
  *
+ *   Copyright 2007 by Apple Inc.
  *   Copyright 2003-2005 by Easy Software Products, all rights reserved.
  *
  *   Redistribution and use in source and binary forms, with or
@@ -55,6 +56,7 @@
  */
 
 #include <scsi/sg.h>
+#include <cups/i18n.h>
 
 
 /*
@@ -95,7 +97,7 @@ print_device(const char *resource,	/* I - SCSI device */
   int		bytes;			/* Number of bytes */
   int		try;			/* Current try */
   sg_io_hdr_t	scsi_req;		/* SCSI request */
-  char		scsi_cmd[6],		/* SCSI command data */
+  unsigned char	scsi_cmd[6],		/* SCSI command data */
 		scsi_sense[32];		/* SCSI sense data */
 #  if defined(HAVE_SIGACTION) && !defined(HAVE_SIGSET)
   struct sigaction action;		/* Actions for POSIX signals */
@@ -108,34 +110,62 @@ print_device(const char *resource,	/* I - SCSI device */
 
   if (strncmp(resource, "/dev/sg", 7) != 0)
   {
-    fprintf(stderr, "ERROR: Bad SCSI device file \"%s\"!\n", resource);
-    return (1);
+    _cupsLangPrintf(stderr, _("ERROR: Bad SCSI device file \"%s\"!\n"),
+                    resource);
+    return (CUPS_BACKEND_STOP);
   }
 
  /*
   * Open the SCSI device file...
   */
 
+  fputs("STATE: +connecting-to-device\n", stderr);
+
   do
   {
     if ((scsi_fd = open(resource, O_RDWR | O_EXCL)) == -1)
     {
+      if (getenv("CLASS") != NULL)
+      {
+       /*
+        * If the CLASS environment variable is set, the job was submitted
+	* to a class and not to a specific queue.  In this case, we want
+	* to abort immediately so that the job can be requeued on the next
+	* available printer in the class.
+	*/
+
+        _cupsLangPuts(stderr,
+	              _("INFO: Unable to contact printer, queuing on next "
+		        "printer in class...\n"));
+
+       /*
+        * Sleep 5 seconds to keep the job from requeuing too rapidly...
+	*/
+
+	sleep(5);
+
+        return (CUPS_BACKEND_FAILED);
+      }
+
       if (errno != EAGAIN && errno != EBUSY)
       {
-	fprintf(stderr, "ERROR: Unable to open SCSI device \"%s\" - %s\n",
-        	resource, strerror(errno));
-	return (1);
+	_cupsLangPrintf(stderr,
+	                _("ERROR: Unable to open device file \"%s\": %s\n"),
+			resource, strerror(errno));
+	return (CUPS_BACKEND_FAILED);
       }
       else
       {
-        fprintf(stderr, "INFO: SCSI device \"%s\" busy; retrying...\n",
-	        resource);
+        _cupsLangPuts(stderr,
+	              _("INFO: Printer busy; will retry in 30 seconds...\n"));
         sleep(30);
       }
     }
   }
   while (scsi_fd == -1);
 
+  fputs("STATE: -connecting-to-device\n", stderr);
+
  /*
   * Now that we are "connected" to the port, ignore SIGTERM so that we
   * can finish out any page data the driver sends (e.g. to eject the
@@ -193,8 +223,9 @@ print_device(const char *resource,	/* I - SCSI device */
 	if (ioctl(scsi_fd, SG_IO, &scsi_req) < 0 ||
             scsi_req.status != 0)
         {
-	  fprintf(stderr, "WARNING: SCSI command timed out (%d); retrying...\n",
-	          scsi_req.status);
+	  _cupsLangPrintf(stderr,
+			  _("WARNING: SCSI command timed out (%d); "
+			    "retrying...\n"), scsi_req.status);
           sleep(try + 1);
 	}
 	else
@@ -202,10 +233,10 @@ print_device(const char *resource,	/* I - SCSI device */
 
       if (try >= 10)
       {
-	fprintf(stderr, "ERROR: Unable to send print data (%d)\n",
-	        scsi_req.status);
+	_cupsLangPrintf(stderr, _("ERROR: Unable to send print data (%d)\n"),
+			scsi_req.status);
         close(scsi_fd);
-	return (1);
+	return (CUPS_BACKEND_FAILED);
       }
     }
 
@@ -218,11 +249,11 @@ print_device(const char *resource,	/* I - SCSI device */
 
   close(fd);
 
-  return (0);
+  return (CUPS_BACKEND_OK);
 }
 #endif /* !SG_DXFER_TO_DEV */
 
 
 /*
- * End of "$Id: scsi-linux.c,v 1.5 2005/01/03 19:29:44 mike Exp $".
+ * End of "$Id: scsi-linux.c 7721 2008-07-11 22:48:49Z mike $".
  */
