@@ -1,5 +1,5 @@
 /*
- * "$Id: socket.c 6911 2007-09-04 20:35:08Z mike $"
+ * "$Id: socket.c 8170 2008-12-08 21:08:49Z mike $"
  *
  *   AppSocket backend for the Common UNIX Printing System (CUPS).
  *
@@ -73,7 +73,9 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
 		sep;			/* Option separator */
   int		print_fd;		/* Print file */
   int		copies;			/* Number of copies to print */
-  time_t	start_time;		/* Time of first connect */
+  time_t	start_time,		/* Time of first connect */
+		current_time,		/* Current time */
+		wait_time;		/* Time to wait before shutting down socket */
   int		recoverable;		/* Recoverable error shown? */
   int		contimeout;		/* Connection timeout */
   int		waiteof;		/* Wait for end-of-file? */
@@ -385,10 +387,13 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
   }
 
  /*
-  * Get any pending back-channel data...
+  * Wait up to 5 seconds to get any pending back-channel data...
   */
 
-  while (wait_bc(device_fd, 5) > 0);
+  wait_time = time(NULL) + 5;
+  while (wait_time >= time(&current_time))
+    if (wait_bc(device_fd, wait_time - current_time) <= 0)
+      break;
 
   if (waiteof)
   {
@@ -466,6 +471,7 @@ side_cb(int print_fd,			/* I - Print file */
         break;
 
     case CUPS_SC_CMD_GET_BIDI :
+	status  = CUPS_SC_STATUS_OK;
         data[0] = use_bc;
         datalen = 1;
         break;
@@ -533,5 +539,5 @@ wait_bc(int device_fd,			/* I - Socket */
 
 
 /*
- * End of "$Id: socket.c 6911 2007-09-04 20:35:08Z mike $".
+ * End of "$Id: socket.c 8170 2008-12-08 21:08:49Z mike $".
  */
