@@ -1,5 +1,5 @@
 /*
- * "$Id: usb-unix.c 7220 2008-01-14 22:12:58Z mike $"
+ * "$Id: usb-unix.c 8170 2008-12-08 21:08:49Z mike $"
  *
  *   USB port backend for the Common UNIX Printing System (CUPS).
  *
@@ -80,6 +80,14 @@ print_device(const char *uri,		/* I - Device URI */
 
     use_bc = 0;
 
+#elif defined(__sun)
+   /*
+    * CUPS STR #3028: Solaris' usbprn driver apparently does not support
+    * select() or poll(), so we can't support backchannel...
+    */
+
+    use_bc = 0;
+
 #else
    /*
     * Disable backchannel data when printing to Brother, Canon, or
@@ -173,7 +181,17 @@ print_device(const char *uri,		/* I - Device URI */
       lseek(print_fd, 0, SEEK_SET);
     }
 
+#ifdef __sun
+   /*
+    * CUPS STR #3028: Solaris' usbprn driver apparently does not support
+    * select() or poll(), so we can't support the sidechannel either...
+    */
+
+    tbytes = backendRunLoop(print_fd, device_fd, use_bc, NULL);
+
+#else
     tbytes = backendRunLoop(print_fd, device_fd, use_bc, side_cb);
+#endif /* __sun */
 
     if (print_fd != 0 && tbytes >= 0)
       _cupsLangPrintf(stderr,
@@ -414,11 +432,10 @@ open_device(const char *uri,		/* I - Device URI */
       */
 
       if (busy)
-      {
 	_cupsLangPuts(stderr,
 	              _("INFO: Printer busy; will retry in 5 seconds...\n"));
-	sleep(5);
-      }
+
+      sleep(5);
     }
   }
 #elif defined(__sun) && defined(ECPPIOC_GETDEVID)
@@ -575,6 +592,7 @@ side_cb(int print_fd,			/* I - Print file */
         break;
 
     case CUPS_SC_CMD_GET_BIDI :
+	status  = CUPS_SC_STATUS_OK;
         data[0] = use_bc;
         datalen = 1;
         break;
@@ -606,5 +624,5 @@ side_cb(int print_fd,			/* I - Print file */
 
 
 /*
- * End of "$Id: usb-unix.c 7220 2008-01-14 22:12:58Z mike $".
+ * End of "$Id: usb-unix.c 8170 2008-12-08 21:08:49Z mike $".
  */
