@@ -1,9 +1,9 @@
 /*
- * "$Id: encode.c 7262 2008-01-28 23:10:10Z mike $"
+ * "$Id: encode.c 8296 2009-01-28 16:49:56Z mike $"
  *
  *   Option encoding routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 2007 by Apple Inc.
+ *   Copyright 2007-2009 by Apple Inc.
  *   Copyright 1997-2007 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -51,6 +51,7 @@ static const _ipp_option_t ipp_options[] =
   { 0, "brightness-default",	IPP_TAG_INTEGER,	IPP_TAG_PRINTER },
   { 0, "columns",		IPP_TAG_INTEGER,	IPP_TAG_JOB },
   { 0, "columns-default",	IPP_TAG_INTEGER,	IPP_TAG_PRINTER },
+  { 0, "compression",		IPP_TAG_KEYWORD,	IPP_TAG_OPERATION },
   { 0, "copies",		IPP_TAG_INTEGER,	IPP_TAG_JOB },
   { 0, "copies-default",	IPP_TAG_INTEGER,	IPP_TAG_PRINTER },
   { 0, "document-format",	IPP_TAG_MIMETYPE,	IPP_TAG_OPERATION },
@@ -63,10 +64,12 @@ static const _ipp_option_t ipp_options[] =
   { 0, "gamma-default",		IPP_TAG_INTEGER,	IPP_TAG_PRINTER },
   { 0, "hue",			IPP_TAG_INTEGER,	IPP_TAG_JOB },
   { 0, "hue-default",		IPP_TAG_INTEGER,	IPP_TAG_PRINTER },
-  { 0, "job-k-limit",		IPP_TAG_INTEGER,	IPP_TAG_JOB },
-  { 0, "job-page-limit",	IPP_TAG_INTEGER,	IPP_TAG_JOB },
+  { 0, "job-k-limit",		IPP_TAG_INTEGER,	IPP_TAG_PRINTER },
+  { 0, "job-page-limit",	IPP_TAG_INTEGER,	IPP_TAG_PRINTER },
   { 0, "job-priority",		IPP_TAG_INTEGER,	IPP_TAG_JOB },
-  { 0, "job-quota-period",	IPP_TAG_INTEGER,	IPP_TAG_JOB },
+  { 0, "job-quota-period",	IPP_TAG_INTEGER,	IPP_TAG_PRINTER },
+  { 1, "job-sheets",		IPP_TAG_NAME,		IPP_TAG_JOB },
+  { 1, "job-sheets-default",	IPP_TAG_NAME,		IPP_TAG_PRINTER },
   { 0, "job-uuid",		IPP_TAG_URI,		IPP_TAG_JOB },
   { 0, "landscape",		IPP_TAG_BOOLEAN,	IPP_TAG_JOB },
   { 1, "media",			IPP_TAG_KEYWORD,	IPP_TAG_JOB },
@@ -584,5 +587,5 @@ compare_ipp_options(_ipp_option_t *a,	/* I - First option */
 
 
 /*
- * End of "$Id: encode.c 7262 2008-01-28 23:10:10Z mike $".
+ * End of "$Id: encode.c 8296 2009-01-28 16:49:56Z mike $".
  */
