@@ -1,9 +1,9 @@
 /*
- * "$Id: http-addrlist.c 7222 2008-01-16 22:20:33Z mike $"
+ * "$Id: http-addrlist.c 8246 2009-01-14 18:02:01Z mike $"
  *
  *   HTTP address list routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 2007-2008 by Apple Inc.
+ *   Copyright 2007-2009 by Apple Inc.
  *   Copyright 1997-2007 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -27,6 +27,10 @@
 #include "globals.h"
 #include "debug.h"
 #include <stdlib.h>
+#include <errno.h>
+#ifdef HAVE_RESOLV_H
+#  include <resolv.h>
+#endif /* HAVE_RESOLV_H */
 
 
 /*
@@ -174,6 +178,8 @@ httpAddrGetList(const char *hostname,	/* I - Hostname, IP address, or NULL for p
   http_addrlist_t	*first,		/* First address in list */
 			*addr,		/* Current address in list */
 			*temp;		/* New address */
+  _cups_globals_t	*cg = _cupsGlobals();
+					/* Global data */
 
 
 #ifdef DEBUG
@@ -189,6 +195,28 @@ httpAddrGetList(const char *hostname,	/* I - Hostname, IP address, or NULL for p
 	     family == AF_INET ? "INET" : "???", service);
 #endif /* DEBUG */
 
+#ifdef HAVE_RES_INIT
+ /*
+  * STR #2920: Initialize resolver after failure in cups-polld
+  *
+  * If the previous lookup failed, re-initialize the resolver to prevent
+  * temporary network errors from persisting.  This *should* be handled by
+  * the resolver libraries, but apparently the glibc folks do not agree.
+  *
+  * We set a flag at the end of this function if we encounter an error that
+  * requires reinitialization of the resolver functions.  We then call
+  * res_init() if the flag is set on the next call here or in httpAddrLookup().
+  */
+
+  if (cg->need_res_init)
+  {
+    res_init();
+
+    cg->need_res_init = 0;
+  }
+#endif /* HAVE_RES_INIT */
+
+
  /*
   * Lookup the address the best way we can...
   */
@@ -210,6 +238,7 @@ httpAddrGetList(const char *hostname,	/* I - Hostname, IP address, or NULL for p
   }
   else
 #endif /* AF_LOCAL */
+  if (!hostname || strcasecmp(hostname, "localhost"))
   {
 #ifdef HAVE_GETADDRINFO
     struct addrinfo	hints,		/* Address lookup hints */
@@ -218,6 +247,8 @@ httpAddrGetList(const char *hostname,	/* I - Hostname, IP address, or NULL for p
     char		ipv6[1024],	/* IPv6 address */
 			*ipv6zone;	/* Pointer to zone separator */
     int			ipv6len;	/* Length of IPv6 address */
+    int			error;		/* getaddrinfo() error */
+
 
    /*
     * Lookup the address as needed...
@@ -269,7 +300,7 @@ httpAddrGetList(const char *hostname,	/* I - Hostname, IP address, or NULL for p
       }
     }
 
-    if (!getaddrinfo(hostname, service, &hints, &results))
+    if ((error = getaddrinfo(hostname, service, &hints, &results)) == 0)
     {
      /*
       * Copy the results to our own address list structure...
@@ -315,6 +346,9 @@ httpAddrGetList(const char *hostname,	/* I - Hostname, IP address, or NULL for p
 
       freeaddrinfo(results);
     }
+    else if (error == EAI_FAIL)
+      cg->need_res_init = 1;
+
 #else
     if (hostname)
     {
@@ -431,6 +465,8 @@ httpAddrGetList(const char *hostname,	/* I - Hostname, IP address, or NULL for p
 	  addr = temp;
 	}
       }
+      else if (h_errno == NO_RECOVERY)
+        cg->need_res_init = 1;
     }
 #endif /* HAVE_GETADDRINFO */
   }
@@ -439,7 +475,7 @@ httpAddrGetList(const char *hostname,	/* I - Hostname, IP address, or NULL for p
   * Detect some common errors and handle them sanely...
   */
 
-  if (!addr && (!hostname || !strcmp(hostname, "localhost")))
+  if (!addr && (!hostname || !strcasecmp(hostname, "localhost")))
   {
     struct servent	*port;		/* Port number for service */
     int			portnum;	/* Port number */
@@ -468,7 +504,7 @@ httpAddrGetList(const char *hostname,	/* I - Hostname, IP address, or NULL for p
     else
       return (NULL);
 
-    if (hostname && !strcmp(hostname, "localhost"))
+    if (hostname && !strcasecmp(hostname, "localhost"))
     {
      /*
       * Unfortunately, some users ignore all of the warnings in the
@@ -528,8 +564,6 @@ httpAddrGetList(const char *hostname,	/* I - Hostname, IP address, or NULL for p
 
         if (addr)
 	  addr->next = temp;
-	else
-          addr = temp;
       }
     }
     else if (!hostname)
@@ -583,8 +617,6 @@ httpAddrGetList(const char *hostname,	/* I - Hostname, IP address, or NULL for p
 
         if (addr)
 	  addr->next = temp;
-	else
-          addr = temp;
       }
     }
   }
@@ -598,5 +630,5 @@ httpAddrGetList(const char *hostname,	/* I - Hostname, IP address, or NULL for p
 
 
 /*
- * End of "$Id: http-addrlist.c 7222 2008-01-16 22:20:33Z mike $".
+ * End of "$Id: http-addrlist.c 8246 2009-01-14 18:02:01Z mike $".
  */
