@@ -1,5 +1,5 @@
 /*
- * "$Id: ipp.c 7323 2008-02-16 00:45:36Z mike $"
+ * "$Id: ipp.c 8275 2009-01-21 23:43:53Z mike $"
  *
  *   Internet Printing Protocol support functions for the Common UNIX
  *   Printing System (CUPS).
@@ -1049,7 +1049,7 @@ ippReadIO(void       *src,		/* I - Data source */
           * Get the request header...
 	  */
 
-          if ((n = (*cb)(src, buffer, 8)) < 8)
+          if ((*cb)(src, buffer, 8) < 8)
 	  {
 	    DEBUG_printf(("ippReadIO: Unable to read header (%d bytes read)!\n", n));
 	    return (IPP_ERROR);
@@ -1556,6 +1556,12 @@ ippReadIO(void       *src,		/* I - Data source */
 		  return (IPP_ERROR);
 		}
 
+		if (!value)
+		{
+		  DEBUG_puts("ippReadIO: NULL value!");
+		  return (IPP_ERROR);
+		}
+
                 value->unknown.length = n;
 	        if (n > 0)
 		{
@@ -2828,7 +2834,18 @@ ipp_read_http(http_t      *http,	/* I - Client connection */
 	}
       }
 
-      if ((bytes = httpRead2(http, (char *)buffer, length - tbytes)) <= 0)
+      if ((bytes = httpRead2(http, (char *)buffer, length - tbytes)) < 0)
+      {
+#ifdef WIN32
+        break;
+#else
+        if (errno != EAGAIN && errno != EINTR)
+	  break;
+
+	bytes = 0;
+#endif /* WIN32 */
+      }
+      else if (bytes == 0)
         break;
     }
   }
@@ -2904,5 +2921,5 @@ _ipp_free_attr(ipp_attribute_t *attr)	/* I - Attribute to free */
 
 
 /*
- * End of "$Id: ipp.c 7323 2008-02-16 00:45:36Z mike $".
+ * End of "$Id: ipp.c 8275 2009-01-21 23:43:53Z mike $".
  */
