@@ -1,5 +1,5 @@
 /*
- * "$Id: util.c 7224 2008-01-16 23:42:35Z mike $"
+ * "$Id: util.c 8407 2009-03-05 18:44:14Z mike $"
  *
  *   Printing utilities for the Common UNIX Printing System (CUPS).
  *
@@ -504,7 +504,7 @@ cupsGetJobs2(http_t     *http,		/* I - HTTP connection */
     }
   }
   else
-    strcpy(uri, "ipp://localhost/jobs");
+    strcpy(uri, "ipp://localhost/");
 
 
  /*
@@ -772,7 +772,7 @@ cupsGetPPD2(http_t     *http,		/* I - HTTP connection */
                             resource, sizeof(resource), 0))
     return (NULL);
 
-  DEBUG_printf(("Printer hostname=\"%s\", port=%d\n", hostname, port));
+  DEBUG_printf(("cupsGetPPD3: Printer hostname=\"%s\", port=%d\n", hostname, port));
 
  /*
   * Remap local hostname to localhost...
@@ -1253,6 +1253,21 @@ cupsPrintFiles2(http_t        *http,	/* I - HTTP connection */
     ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_NAME, "job-name", NULL,
                  title);
 
+  if (num_files == 1)
+  {
+   /*
+    * Add the original document filename...
+    */
+
+    if ((base = strrchr(files[0], '/')) != NULL)
+      base ++;
+    else
+      base = files[0];
+
+    ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_NAME, "document-name",
+		 NULL, base);
+  }
+
  /*
   * Then add all options...
   */
@@ -1611,6 +1626,16 @@ cups_get_printer_uri(
 		      host, hostsize, port, resource, resourcesize);
       ippDelete(response);
 
+      if (!strncmp(resource, "/classes/", 9))
+      {
+        _cupsSetError(IPP_INTERNAL_ERROR, _("No printer-uri found for class!"));
+
+	*host     = '\0';
+	*resource = '\0';
+
+	return (0);
+      }
+
       return (1);
     }
 
@@ -1618,7 +1643,7 @@ cups_get_printer_uri(
   }
 
   if (cupsLastError() != IPP_NOT_FOUND)
-    _cupsSetError(IPP_INTERNAL_ERROR, "No printer-uri found!");
+    _cupsSetError(IPP_INTERNAL_ERROR, _("No printer-uri found!"));
 
   *host     = '\0';
   *resource = '\0';
@@ -1628,5 +1653,5 @@ cups_get_printer_uri(
 
 
 /*
- * End of "$Id: util.c 7224 2008-01-16 23:42:35Z mike $".
+ * End of "$Id: util.c 8407 2009-03-05 18:44:14Z mike $".
  */
