@@ -1,26 +1,17 @@
 /*
- * "$Id: image-private.h 6158 2006-12-17 01:44:21Z mike $"
+ * "$Id: image-private.h 8510 2009-04-16 18:26:25Z mike $"
  *
  *   Private image library definitions for the Common UNIX Printing
  *   System (CUPS).
  *
+ *   Copyright 2007 by Apple Inc.
  *   Copyright 1993-2006 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  *   This file is subject to the Apple OS-Developed Software exception.
  */
@@ -50,8 +41,8 @@
 
 #  define CUPS_IMAGE_MAX_WIDTH	0x07ffffff
 					/* 2^27-1 to allow for 15-channel data */
-#  define CUPS_IMAGE_MAX_HEIGHT	0x7fffffff
-					/* 2^31-1 */
+#  define CUPS_IMAGE_MAX_HEIGHT	0x3fffffff
+					/* 2^30-1 */
 
 #  define CUPS_TILE_SIZE	256	/* 256x256 pixel tiles */
 #  define CUPS_TILE_MINIMUM	10	/* Minimum number of tiles */
@@ -217,10 +208,11 @@ extern cups_izoom_t	*_cupsImageZoomNew(cups_image_t *img, int xc0, int yc0,
 extern int		_cupsRasterExecPS(cups_page_header2_t *h,
 			                  int *preferred_bits,
 			                  const char *code);
-
+extern void		_cupsRasterAddError(const char *f, ...);
+extern void		_cupsRasterClearError(void);
 
 #endif /* !_CUPS_IMAGE_PRIVATE_H_ */
 
 /*
- * End of "$Id: image-private.h 6158 2006-12-17 01:44:21Z mike $".
+ * End of "$Id: image-private.h 8510 2009-04-16 18:26:25Z mike $".
  */
