@@ -1,5 +1,5 @@
 /*
- * "$Id: cert.h 7319 2008-02-15 23:26:51Z mike $"
+ * "$Id: cert.h 7384 2008-03-20 20:58:16Z mike $"
  *
  *   Authentication certificate definitions for the Common UNIX
  *   Printing System (CUPS).
@@ -48,10 +48,10 @@ extern void		cupsdAddCert(int pid, const char *username,
 			             void *ccache);
 extern void		cupsdDeleteCert(int pid);
 extern void		cupsdDeleteAllCerts(void);
-extern const char	*cupsdFindCert(const char *certificate);
+extern cupsd_cert_t	*cupsdFindCert(const char *certificate);
 extern void		cupsdInitCerts(void);
 
 
 /*
- * End of "$Id: cert.h 7319 2008-02-15 23:26:51Z mike $".
+ * End of "$Id: cert.h 7384 2008-03-20 20:58:16Z mike $".
  */
