@@ -1,9 +1,9 @@
 /*
- * "$Id: classes.c 7224 2008-01-16 23:42:35Z mike $"
+ * "$Id: classes.c 8221 2009-01-09 19:56:00Z mike $"
  *
  *   Printer class routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 2007 by Apple Inc.
+ *   Copyright 2007-2009 by Apple Inc.
  *   Copyright 1997-2007 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -123,9 +123,7 @@ cupsdDeletePrinterFromClass(
     cupsd_printer_t *c,			/* I - Class to delete from */
     cupsd_printer_t *p)			/* I - Printer to delete */
 {
-  int		i;			/* Looping var */
-  cups_ptype_t	type,			/* Class type */
-		oldtype;		/* Old class type */
+  int	i;				/* Looping var */
 
 
  /*
@@ -155,27 +153,10 @@ cupsdDeletePrinterFromClass(
     return;
 
  /*
-  * Recompute the printer type mask as needed...
+  * Update the IPP attributes (have to do this for member-names)...
   */
 
-  if (c->num_printers > 0)
-  {
-    oldtype = c->type;
-    type    = c->type & (CUPS_PRINTER_CLASS | CUPS_PRINTER_IMPLICIT);
-    c->type = ~CUPS_PRINTER_REMOTE;
-
-    for (i = 0; i < c->num_printers; i ++)
-      c->type &= c->printers[i]->type;
-
-    c->type |= type;
-
-   /*
-    * Update the IPP attributes...
-    */
-
-    if (c->type != oldtype)
-      cupsdSetPrinterAttrs(c);
-  }
+  cupsdSetPrinterAttrs(c);
 }
 
 
@@ -210,7 +191,8 @@ cupsdDeletePrinterFromClasses(
        c = (cupsd_printer_t *)cupsArrayNext(ImplicitPrinters))
     if (c->num_printers == 0)
     {
-      cupsArrayRemove(ImplicitPrinters, c);
+      cupsdLogMessage(CUPSD_LOG_DEBUG, "Deleting implicit class \"%s\"...",
+                      c->name);
       cupsdDeletePrinter(c, 0);
     }
 }
@@ -600,7 +582,7 @@ cupsdLoadAllClasses(void)
 	       valueptr ++);
 
 	  if (*valueptr)
-            *valueptr++ = '\0';
+            *valueptr = '\0';
 
 	  cupsdSetString(&p->job_sheets[1], value);
 	}
@@ -939,5 +921,5 @@ cupsdUpdateImplicitClasses(void)
 
 
 /*
- * End of "$Id: classes.c 7224 2008-01-16 23:42:35Z mike $".
+ * End of "$Id: classes.c 8221 2009-01-09 19:56:00Z mike $".
  */
