@@ -1,5 +1,5 @@
 /*
- * "$Id: client.h 8258 2009-01-15 19:12:15Z mike $"
+ * "$Id: client.h 8650 2009-05-16 23:26:27Z mike $"
  *
  *   Client definitions for the Common UNIX Printing System (CUPS) scheduler.
  *
@@ -44,7 +44,8 @@ struct cupsd_client_s
   int			pipe_pid;	/* Pipe process ID (or 0 if not a pipe) */
   int			sent_header,	/* Non-zero if sent HTTP header */
 			got_fields,	/* Non-zero if all fields seen */
-			field_col;	/* Column within line */
+			header_used;	/* Number of header bytes used */
+  char			header[2048];	/* Header from CGI program */
   cups_lang_t		*language;	/* Language to use */
 #ifdef HAVE_SSL
   int			auto_ssl;	/* Automatic test for SSL/TLS */
@@ -130,5 +131,5 @@ extern void	cupsdWriteClient(cupsd_client_t *con);
 
 
 /*
- * End of "$Id: client.h 8258 2009-01-15 19:12:15Z mike $".
+ * End of "$Id: client.h 8650 2009-05-16 23:26:27Z mike $".
  */
