@@ -1,10 +1,10 @@
 /*
- * "$Id: conf.h 7319 2008-02-15 23:26:51Z mike $"
+ * "$Id: conf.h 8513 2009-04-16 19:32:04Z mike $"
  *
  *   Configuration file definitions for the Common UNIX Printing System (CUPS)
  *   scheduler.
  *
- *   Copyright 2007 by Apple Inc.
+ *   Copyright 2007-2009 by Apple Inc.
  *   Copyright 1997-2007 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -45,6 +45,17 @@ typedef enum
 #define PRINTCAP_SOLARIS	1	/* Solaris lpsched format */
 
 
+/*
+ * ServerAlias data...
+ */
+
+typedef struct
+{
+  size_t	namelen;		/* Length of alias name */
+  char		name[1];		/* Alias name */
+} cupsd_alias_t;
+
+
 /*
  * Globals...
  */
@@ -65,7 +76,12 @@ VAR char		*ConfigurationFile	VALUE(NULL),
 					/* Directory for request files */
 			*DocumentRoot		VALUE(NULL);
 					/* Root directory for documents */
-VAR int			ServerNameIsIP		VALUE(0);
+VAR cups_array_t	*ServerAlias		VALUE(NULL);
+					/* Alias names for server */
+VAR int			RemoteAccessEnabled	VALUE(0),
+					/* Are we listening on non-local addresses? */
+			ServerNameIsIP		VALUE(0);
+					/* Is the ServerName an IP address? */
 VAR int			NumSystemGroups		VALUE(0);
 					/* Number of system group names */
 VAR char		*SystemGroups[MAX_SYSTEM_GROUPS]
@@ -233,5 +249,5 @@ extern int	cupsdReadConfiguration(void);
 
 
 /*
- * End of "$Id: conf.h 7319 2008-02-15 23:26:51Z mike $".
+ * End of "$Id: conf.h 8513 2009-04-16 19:32:04Z mike $".
  */
