@@ -1,34 +1,25 @@
 /*
- * "$Id: filter.c 5606 2006-05-30 19:40:34Z mike $"
+ * "$Id: filter.c 6649 2007-07-11 21:46:42Z mike $"
  *
  *   File type conversion routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2006 by Easy Software Products, all rights reserved.
+ *   Copyright 2007 by Apple Inc.
+ *   Copyright 1997-2007 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  * Contents:
  *
- *   mimeAddFilter()   - Add a filter to the current MIME database.
- *   mimeFilter()      - Find the fastest way to convert from one type to
- *                       another.
- *   compare_filters() - Compare two filters...
- *   find_filters()    - Find the filters to convert from one type to another.
- *   lookup()          - Lookup a filter...
+ *   mimeAddFilter()    - Add a filter to the current MIME database.
+ *   mimeFilter()       - Find the fastest way to convert from one type to
+ *                        another.
+ *   mimeFilterLookup() - Lookup a filter...
+ *   compare_filters()  - Compare two filters...
+ *   find_filters()     - Find the filters to convert from one type to another.
  */
 
 /*
@@ -64,7 +55,6 @@ static int		compare_srcs(mime_filter_t *, mime_filter_t *);
 static cups_array_t	*find_filters(mime_t *mime, mime_type_t *src,
 			              mime_type_t *dst, int *cost,
 				      _mime_typelist_t *visited);
-static mime_filter_t	*lookup(mime_t *, mime_type_t *, mime_type_t *);
 
 
 /*
@@ -93,7 +83,7 @@ mimeAddFilter(mime_t      *mime,	/* I - MIME database */
   * destination...
   */
 
-  if ((temp = lookup(mime, src, dst)) != NULL)
+  if ((temp = mimeFilterLookup(mime, src, dst)) != NULL)
   {
    /*
     * Yup, does the existing filter have a higher cost?  If so, copy the
@@ -193,6 +183,25 @@ mimeFilter(mime_t      *mime,		/* I - MIME database */
 }
 
 
+/*
+ * 'mimeFilterLookup()' - Lookup a filter...
+ */
+
+mime_filter_t *				/* O - Filter for src->dst */
+mimeFilterLookup(mime_t      *mime,	/* I - MIME database */
+                 mime_type_t *src,	/* I - Source type */
+                 mime_type_t *dst)	/* I - Destination type */
+{
+  mime_filter_t	key;			/* Key record for filter search */
+
+
+  key.src = src;
+  key.dst = dst;
+
+  return ((mime_filter_t *)cupsArrayFind(mime->filters, &key));
+}
+
+
 /*
  * 'compare_filters()' - Compare two filters...
  */
@@ -260,7 +269,7 @@ find_filters(mime_t           *mime,	/* I - MIME database */
   * See if there is a filter that can convert the files directly...
   */
 
-  if ((current = lookup(mime, src, dst)) != NULL)
+  if ((current = mimeFilterLookup(mime, src, dst)) != NULL)
   {
    /*
     * Got a direct filter!
@@ -392,24 +401,5 @@ find_filters(mime_t           *mime,	/* I - MIME database */
 
 
 /*
- * 'lookup()' - Lookup a filter...
- */
-
-static mime_filter_t *			/* O - Filter for src->dst */
-lookup(mime_t      *mime,		/* I - MIME database */
-       mime_type_t *src,		/* I - Source type */
-       mime_type_t *dst)		/* I - Destination type */
-{
-  mime_filter_t	key;			/* Key record for filter search */
-
-
-  key.src = src;
-  key.dst = dst;
-
-  return ((mime_filter_t *)cupsArrayFind(mime->filters, &key));
-}
-
-
-/*
- * End of "$Id: filter.c 5606 2006-05-30 19:40:34Z mike $".
+ * End of "$Id: filter.c 6649 2007-07-11 21:46:42Z mike $".
  */
