@@ -1,26 +1,17 @@
 /*
- * "$Id: network.c 6090 2006-11-14 16:35:27Z mike $"
+ * "$Id: network.c 8513 2009-04-16 19:32:04Z mike $"
  *
  *   Network interface functions for the Common UNIX Printing System
  *   (CUPS) scheduler.
  *
+ *   Copyright 2007-2009 by Apple Inc.
  *   Copyright 1997-2006 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
  *   "LICENSE" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  * Contents:
  *
@@ -109,8 +100,8 @@ cupsdNetIFUpdate(void)
   cupsd_netif_t		*temp;		/* New interface */
   struct ifaddrs	*addrs,		/* Interface address list */
 			*addr;		/* Current interface address */
-  http_addrlist_t	*saddr;		/* Current server address */
   char			hostname[1024];	/* Hostname for address */
+  size_t		hostlen;	/* Length of hostname */
 
 
  /*
@@ -164,7 +155,7 @@ cupsdNetIFUpdate(void)
     * Try looking up the hostname for the address as needed...
     */
 
-    if (HostNameLookups)
+    if (HostNameLookups || RemoteAccessEnabled)
       httpAddrLookup((http_addr_t *)(addr->ifa_addr), hostname,
                      sizeof(hostname));
     else
@@ -178,25 +169,16 @@ cupsdNetIFUpdate(void)
       if (httpAddrLocalhost((http_addr_t *)(addr->ifa_addr)))
         strcpy(hostname, "localhost");
       else
-      {
-        for (saddr = ServerAddrs; saddr; saddr = saddr->next)
-	  if (httpAddrEqual((http_addr_t *)(addr->ifa_addr), &(saddr->addr)))
-	    break;
-
-	if (saddr)
-          strlcpy(hostname, ServerName, sizeof(hostname));
-	else
-          httpAddrString((http_addr_t *)(addr->ifa_addr), hostname,
-	        	 sizeof(hostname));
-      }
+	httpAddrString((http_addr_t *)(addr->ifa_addr), hostname,
+		       sizeof(hostname));
     }
 
    /*
     * Create a new address element...
     */
 
-    if ((temp = calloc(1, sizeof(cupsd_netif_t) +
-                          strlen(hostname))) == NULL)
+    hostlen = strlen(hostname);
+    if ((temp = calloc(1, sizeof(cupsd_netif_t) + hostlen)) == NULL)
       break;
 
    /*
@@ -204,6 +186,7 @@ cupsdNetIFUpdate(void)
     */
 
     strlcpy(temp->name, addr->ifa_name, sizeof(temp->name));
+    temp->hostlen = hostlen;
     strcpy(temp->hostname, hostname);	/* Safe because hostname is allocated */
 
     if (addr->ifa_addr->sa_family == AF_INET)
@@ -254,24 +237,29 @@ cupsdNetIFUpdate(void)
       else if (addr->ifa_addr->sa_family == AF_INET &&
                lis->address.addr.sa_family == AF_INET &&
                (lis->address.ipv4.sin_addr.s_addr &
-	           temp->mask.ipv4.sin_addr.s_addr) ==
-	               temp->address.ipv4.sin_addr.s_addr)
+	        temp->mask.ipv4.sin_addr.s_addr) ==
+	           (temp->address.ipv4.sin_addr.s_addr &
+		    temp->mask.ipv4.sin_addr.s_addr))
         match = 1;
 #ifdef AF_INET6
       else if (addr->ifa_addr->sa_family == AF_INET6 &&
                lis->address.addr.sa_family == AF_INET6 &&
                (lis->address.ipv6.sin6_addr.s6_addr[0] &
-	           temp->mask.ipv6.sin6_addr.s6_addr[0]) ==
-	               temp->address.ipv6.sin6_addr.s6_addr[0] &&
+	        temp->mask.ipv6.sin6_addr.s6_addr[0]) ==
+		   (temp->address.ipv6.sin6_addr.s6_addr[0] &
+		    temp->mask.ipv6.sin6_addr.s6_addr[0]) &&
                (lis->address.ipv6.sin6_addr.s6_addr[1] &
-	           temp->mask.ipv6.sin6_addr.s6_addr[1]) ==
-	               temp->address.ipv6.sin6_addr.s6_addr[1] &&
+	        temp->mask.ipv6.sin6_addr.s6_addr[1]) ==
+		   (temp->address.ipv6.sin6_addr.s6_addr[1] &
+		    temp->mask.ipv6.sin6_addr.s6_addr[1]) &&
                (lis->address.ipv6.sin6_addr.s6_addr[2] &
-	           temp->mask.ipv6.sin6_addr.s6_addr[2]) ==
-	               temp->address.ipv6.sin6_addr.s6_addr[2] &&
+	        temp->mask.ipv6.sin6_addr.s6_addr[2]) ==
+		   (temp->address.ipv6.sin6_addr.s6_addr[2] &
+		    temp->mask.ipv6.sin6_addr.s6_addr[2]) &&
                (lis->address.ipv6.sin6_addr.s6_addr[3] &
-	           temp->mask.ipv6.sin6_addr.s6_addr[3]) ==
-	               temp->address.ipv6.sin6_addr.s6_addr[3])
+	        temp->mask.ipv6.sin6_addr.s6_addr[3]) ==
+		   (temp->address.ipv6.sin6_addr.s6_addr[3] &
+		    temp->mask.ipv6.sin6_addr.s6_addr[3]))
         match = 1;
 #endif /* AF_INET6 */
 
@@ -293,8 +281,8 @@ cupsdNetIFUpdate(void)
 
     cupsArrayAdd(NetIFList, temp);
 
-    cupsdLogMessage(CUPSD_LOG_DEBUG, "cupsdNetIFUpdate: \"%s\" = %s...",
-                    temp->name, temp->hostname);
+    cupsdLogMessage(CUPSD_LOG_DEBUG, "cupsdNetIFUpdate: \"%s\" = %s:%d",
+                    temp->name, temp->hostname, temp->port);
   }
 
   freeifaddrs(addrs);
@@ -314,5 +302,5 @@ compare_netif(cupsd_netif_t *a,		/* I - First network interface */
 
 
 /*
- * End of "$Id: network.c 6090 2006-11-14 16:35:27Z mike $".
+ * End of "$Id: network.c 8513 2009-04-16 19:32:04Z mike $".
  */
