@@ -5,7 +5,7 @@ import javax.swing.*;
 import java.net.URL;
 import java.net.*;
 import java.io.*;
-import org.cups.*;
+import com.easysw.cups.*;
 
 public class GLPprinters implements ActionListener 
 {
