@@ -2,28 +2,19 @@ package com.easysw.cups;
 
 /**
  * @version 1.2 26-FEB-2006
- * @author  Easy Software Products
+ * @author  Apple Inc.
  *
  *   Internet Printing Protocol definitions for the Common UNIX Printing
  *   System (CUPS).
  *
+ *   Copyright 2007-2008 by Apple Inc.
  *   Copyright 1997-2006 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org/
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  */
 
 /**
@@ -840,7 +831,7 @@ public class Cups
 
       a = new IPPAttribute( IPPDefs.TAG_OPERATION, IPPDefs.TAG_CHARSET,
                             "attributes-charset" );
-      a.addString( "", "iso-8859-1" );  
+      a.addString( "", "utf-8" );  
       ipp.addAttribute(a);
             
 
@@ -1023,7 +1014,7 @@ public class Cups
 
       a = new IPPAttribute( IPPDefs.TAG_OPERATION, IPPDefs.TAG_CHARSET,
                             "attributes-charset" );
-      a.addString( "", "iso-8859-1" );  
+      a.addString( "", "utf-8" );  
       ipp.addAttribute(a);
             
 
@@ -1100,7 +1091,7 @@ public class Cups
 
       a = new IPPAttribute( IPPDefs.TAG_OPERATION, IPPDefs.TAG_CHARSET,
                             "attributes-charset" );
-      a.addString( "", "iso-8859-1" );  
+      a.addString( "", "utf-8" );  
       ipp.addAttribute(a);
             
 
@@ -1164,7 +1155,7 @@ public class Cups
 
       a = new IPPAttribute( IPPDefs.TAG_OPERATION, IPPDefs.TAG_CHARSET,
                             "attributes-charset" );
-      a.addString( "", "iso-8859-1" );  
+      a.addString( "", "utf-8" );  
       ipp.addAttribute(a);
             
 
@@ -1238,7 +1229,7 @@ public class Cups
 
       a = new IPPAttribute( IPPDefs.TAG_OPERATION, IPPDefs.TAG_CHARSET,
                             "attributes-charset" );
-      a.addString( "", "iso-8859-1" );  
+      a.addString( "", "utf-8" );  
       ipp.addAttribute(a);
 
       // ------------
@@ -1324,7 +1315,7 @@ public class Cups
 
       a = new IPPAttribute( IPPDefs.TAG_OPERATION, IPPDefs.TAG_CHARSET,
                             "attributes-charset" );
-      a.addString( "", "iso-8859-1" );  
+      a.addString( "", "utf-8" );  
       ipp.addAttribute(a);
 
       // ------------
@@ -1384,7 +1375,7 @@ public class Cups
 
       a = new IPPAttribute( IPPDefs.TAG_OPERATION, IPPDefs.TAG_CHARSET,
                             "attributes-charset" );
-      a.addString( "", "iso-8859-1" );  
+      a.addString( "", "utf-8" );  
       ipp.addAttribute(a);
             
 
