@@ -1,25 +1,16 @@
 /*
- * "$Id: cancel.c 5926 2006-09-05 20:45:47Z mike $"
+ * "$Id: cancel.c 7721 2008-07-11 22:48:49Z mike $"
  *
  *   "cancel" command for the Common UNIX Printing System (CUPS).
  *
+ *   Copyright 2007-2008 by Apple Inc.
  *   Copyright 1997-2006 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  * Contents:
  *
@@ -68,7 +59,6 @@ main(int  argc,				/* I - Number of command-line arguments */
 
   op        = IPP_CANCEL_JOB;
   purge     = 0;
-  job_id    = 0;
   dest      = NULL;
   user      = NULL;
   http      = NULL;
@@ -392,5 +382,5 @@ main(int  argc,				/* I - Number of command-line arguments */
 
 
 /*
- * End of "$Id: cancel.c 5926 2006-09-05 20:45:47Z mike $".
+ * End of "$Id: cancel.c 7721 2008-07-11 22:48:49Z mike $".
  */
