@@ -1,5 +1,5 @@
 /*
- * "$Id: lpstat.c 6649 2007-07-11 21:46:42Z mike $"
+ * "$Id: lpstat.c 8146 2008-11-19 19:50:56Z mike $"
  *
  *   "lpstat" command for the Common UNIX Printing System (CUPS).
  *
@@ -1432,18 +1432,23 @@ show_jobs(http_t     *http,		/* I - HTTP connection to server */
   *
   *    attributes-charset
   *    attributes-natural-language
-  *    job-uri
+  *    printer-uri
   *    requested-attributes
+  *    requesting-user-name
+  *    which-jobs
   */
 
   request = ippNewRequest(IPP_GET_JOBS);
 
+  ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_URI, "printer-uri",
+               NULL, "ipp://localhost/");
+
   ippAddStrings(request, IPP_TAG_OPERATION, IPP_TAG_KEYWORD,
                 "requested-attributes", sizeof(jattrs) / sizeof(jattrs[0]),
 		NULL, jattrs);
 
-  ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_URI, "job-uri",
-               NULL, "ipp://localhost/jobs/");
+  ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_NAME, "requesting-user-name",
+               NULL, cupsUser());
 
   ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_KEYWORD, "which-jobs",
                NULL, which);
@@ -1979,22 +1984,30 @@ show_printers(http_t      *http,	/* I - HTTP connection to server */
 	    * Get the current active job on this queue...
 	    */
 
+            ipp_jstate_t jobstate = IPP_JOB_PENDING;
 	    jobid = 0;
 
 	    for (jobattr = jobs->attrs; jobattr; jobattr = jobattr->next)
 	    {
 	      if (!jobattr->name)
-	        continue;
+	      {
+	        if (jobstate == IPP_JOB_PROCESSING)
+		  break;
+	        else
+		  continue;
+              }
 
 	      if (!strcmp(jobattr->name, "job-id") &&
 	          jobattr->value_tag == IPP_TAG_INTEGER)
 		jobid = jobattr->values[0].integer;
               else if (!strcmp(jobattr->name, "job-state") &&
-	               jobattr->value_tag == IPP_TAG_ENUM &&
-		       jobattr->values[0].integer == IPP_JOB_PROCESSING)
-		break;
+	               jobattr->value_tag == IPP_TAG_ENUM)
+		jobstate = jobattr->values[0].integer;
 	    }
 
+            if (jobstate != IPP_JOB_PROCESSING)
+	      jobid = 0;
+
             ippDelete(jobs);
 	  }
         }
@@ -2258,5 +2271,5 @@ show_scheduler(http_t *http)	/* I - HTTP connection to server */
 
 
 /*
- * End of "$Id: lpstat.c 6649 2007-07-11 21:46:42Z mike $".
+ * End of "$Id: lpstat.c 8146 2008-11-19 19:50:56Z mike $".
  */
